classdef EColiFBFPExtendedModel < EColiFBFPModel
    % EColiFBFPExtendedModel
    %   Ecoli fed-batch fermentation process, Monod Kinetics with O2 
    
    properties
        % model parameters
        tdo1, DOd, DO0, DOo, k3, Kla
    end
    
    methods
        %% Constructor
        function this = EColiFBFPExtendedModel(modelName, path, selectedEquationsList,...
                                                selectedKinetics, pathToExperimentalData)
            if nargin ~= 5
                 exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                 throw(exception);
            end
            
            this = this@EColiFBFPModel(modelName, path, selectedEquationsList,...
                            selectedKinetics, pathToExperimentalData);
            
%             this.executesInOneStep = 0;
            
            % set the number of variables and their ranges
            this = this.initializeProblemParameters();
            this.nvar = length(this.problemParametersList);
            
        end
        
        %% convert to base model
%         function basicModel = extractBasicModel(this)
%             
%             configuration = Configurations();
%             basicModel = EColiFBFPModel(configuration.BASIC_ECOLI_FB_FP_MONOD, ...
%                                                     this.path, this.equationDescriptionsList, ...
%                                                     this.kinetics, this.pathToExperimentalData);
%                                                 
%             basicModel.estimations = this.estimations;                                    
%         end
    end
    
    methods (Access = protected) 
        %% initialize the problem parameters' list
        function this = initializeProblemParameters(this)
            
            this = this.initializeProblemParameters@EColiFBFPModel();
            
            DOo = Parameter("O2*", 20.5, 20.5, 21.5);
            this.problemParametersList(4) = DOo;
            
            k3 = Parameter("1/Yox", 60, 60, 63);
            this.problemParametersList(5) = k3;
            
            Kla = Parameter("Kla", 150, 150, 160);
            this.problemParametersList(6) = Kla;
            
        end
    end
    
    % virtual methods, to be overriden in the subclasses
    % loading data from the experimental data files
    % executing the process
    methods (Access = protected)
    
        function [this, data] = loadData(this) 
            
            [this, data] = loadData@EColiFBFPModel(this);
            
            this.tdo1 = data(:, 6);
            this.tdo1 = this.tdo1(1:292);
            
            this.DOd = data(:, 7);
            this.DO0 = this.DOd(1);
            this.DOd = this.DOd(1:292);
        end
        
         %% set the initial parameters (model and block paraeters)
        function this = initializeModelParametersList(this)
            
            this = initializeModelParametersList@EColiFBFPModel(this);
            
            % set the rest of the parameters in module O2
            doo = ModelParameter('Value', '/O2/DOo');
            doo.value = mat2str(this.DOo);
            this.modelVariablesList(end + 1) = doo;
            
            K3 = ModelParameter('Value', '/O2/K3');
            K3.value = mat2str(this.k3);
            this.modelVariablesList(end + 1) = K3;
            
            kla = ModelParameter('Value', '/O2/Kla');
            kla.value = mat2str(this.Kla);
            this.modelVariablesList(end + 1) = kla;
            
            doInitialCondition = ModelParameter('InitialCondition', '/O2/DO');
            doInitialCondition.value = num2str(this.DO0);
            this.modelVariablesList(end + 1) = doInitialCondition;
        end 
        
        function this = setValuesForTheProblemParameters(this, population)
            this = setValuesForTheProblemParameters@EColiFBFPModel(this, population);
            
            this.DOo    = population(:, 4);
            this.k3     = population(:, 5); 
            this.Kla    = population(:, 6); 
        end
    
        %% interpolation of the results
        function [Xmod, Smod, Xexp, Sexp, DOmod, DOexp, T] = interpolate(this, output, solutionSize)

            [Xmod, Smod, Xexp, Sexp] = this.interpolate@EColiFBFPModel(output, solutionSize);
            
            DOout = output.O2out;
            T = output.tout;

            DOmod = interp1(T, DOout, this.tdo1);
            DOexp = rep(this.DOd, [1 solutionSize]);
        end
    end
   
    methods (Access = public)
        %% set estimations for the parameters evaluated on the first stage
%         function this = setFirstStageParameterEstimations(this, bestSolution)
%             
%             this = setFirstStageParameterEstimations@EColiFBFPModel(this, bestSolution);
%             
%             % remove the estimated parameters from the list
% %             this.problemParametersList = this.problemParametersList(4: end);
% %             this.nvar = length(this.problemParametersList);
%         end
        
        function this = setParameterEstimations(this, bestSolution)
            this.estimations = [this.estimations bestSolution(3: length(bestSolution))];
            
            this.DOo = this.estimations(4);
            this.k3 = this.estimations(5);
            this.Kla = this.estimations(6);
        end
        
        function error = evaluate(this, solution)
            % start the simulation of the model for the current population
            [this, output] = this.execute(solution);

            [Xmod, Smod, Xexp, Sexp, DOmod, DOexp, T] = this.interpolate(output, size(solution));
            
            Ex  = (Xmod - Xexp); 
            Es  = (Smod - Sexp); 
            EDO = (DOmod - DOexp);
            
            error = sum(Ex.*Ex) + sum(Es.*Es) + sum(EDO.*EDO);
            
            EColiFBFPPlot.displayData(output.tout, output.Xout, output.Sout, this.t, this.Xd, this.Sd);
            EColiFBFPPlot.displayO2Data(output.O2out, output.tout, this.DOd, this.tdo1);
         end
        
         %% additional processing of the result
        function [this, bestSolution, meanError, fileName] = process(this, result)

            % find the min error and the corresponding estimations for
            % the problem parameters

            % the estimations of the errors are stored in the first
            % column
            [~, minIndex] = min(result(:, 1));

            bestSolution = result(minIndex, 3: size(result, 2));            
            meanError = mean(result(:, 1));

            % start simulation for the problem parameters' values
            % corresponding to the min error     
            
            % basicModel = extractBasicModel(this);
            
            % empty array for the results in order to use
            % the estimates for the parameters
            % basicModel.process([]);
            
            % TODO: move these operations to a different class,
            % change the method to a non-static
            [this, output] = this.execute(bestSolution);

            % store the output
            fileName = EColiFBFPSaver.saveExtended(this, output);
            
            % use these data for the plot
            EColiFBFPPlot.displayDataOnNamedPlot(output.tout, output.Xout, output.Sout, this.t, this.Xd, this.Sd, "Best solution");
            EColiFBFPPlot.displayO2DataOnNamedFigure(output.O2out, output.tout, this.DOd, this.tdo1, "Best solution");
        end
    end
    
    methods (Access = public, Static)
        %% generate char array with the results
        function solutionString = resultToString(bestSolution, meanError)
            
            muMax = bestSolution(1);
            ks = bestSolution(2);
            k1 = bestSolution(3);
            
            DOo = bestSolution(4);
            k3 = bestSolution(5);
            Kla = bestSolution(6);
            
            solutionString = ...
                ['mumax = ' num2str(muMax, '%.2f') ...
                 ' ks = ' num2str(ks, '%.2f') ...
                 ' 1/Yxs = ' num2str(k1, '%.2f') ...
                 'O2* = ' num2str(DOo, '%.2f') ...
                 ' 1/Yox = ' num2str(k3, '%.2f') ...
                 ' K1a = ' num2str(Kla, '%.2f') ...
                 ', meanJ = ' num2str(meanError, '%.2f')];
            
        end
        
        %% the text shows that this is the basic model
        function description = toString()
            description = 'E';
        end
    end
end

