classdef EColiFBFPModel < ProcessModel
    % EColiFBFermentationProcess describes the basic model
    %   with the basic list of model parameters
    
    properties
        % model parameters
        t, Xd, Sd, F, V, So, X0, S0, Mmax, Ks, K1
        
        % indicator for plotting the data for the first time
        dataPlotted     logical = false;
        
        % estimations of the parameters
        estimations = [];
    end
    
    methods %(Access = public)
        %% constructor
        function this = EColiFBFPModel(modelName, path, selectedEquationsList, ...
                                        selectedKinetics, pathToExperimentalData)
            if nargin ~= 5
                 exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                 throw(exception);
            end
            
            this = this@ProcessModel(modelName, path, selectedEquationsList, selectedKinetics);
            
            % set the number of variables and their ranges
            this = this.initializeProblemParameters();
            this.nvar = length(this.problemParametersList);
            
            this.pathToExperimentalData = pathToExperimentalData;
        end
        
        %% set the estimations of the problem parameters
        function this = set.estimations(this, problemEstimations)
            if ~isempty(problemEstimations)
                this.estimations = problemEstimations;

                this.Mmax = this.estimations(1);
                this.Ks = this.estimations(2);
                this.K1 = this.estimations(3);
            end
        end
    end
    
    methods (Access = protected) 
        %% initialize the problem parameters' list
        function this = initializeProblemParameters(this)
            
            mumax = Parameter("mumax", 0.45, 0.45, 0.52);
            this.problemParametersList(1) = mumax;
            
            ks = Parameter("ks", 0.005, 0.005, 0.05);
            this.problemParametersList(2) = ks;
            
            k1 = Parameter("1/Yxs", 1.8, 1.8, 2.05);
            this.problemParametersList(3) = k1;
            
        end
    end
    
    % virtual methods, to be overloaded in the subclasses
    % loading data from the experimental data files
    % executing the process
    methods (Access = protected)
    
        function [this, data] = loadData(this) 
            
            % TODO: some validation of the input data
            expData = EColiFBFPReader.loadExperimentalData(this.pathToExperimentalData);
            
            this.t  = expData(:, 1);
            this.Sd = expData(:, 5);
            this.Xd = expData(:, 4); 
            this.F  = expData(:, 3);
            this.V  = expData(:, 2);

            this.So = 100;

            this.X0 = this.Xd(1);
            this.S0 = this.Sd(1);

            data = expData;
        end
        
         %% set the initial parameters (model and block parameters)
        function this = initializeModelParametersList(this)
            
            startTime = ModelParameter('StartTime', '');
            startTime.value = num2str(this.t(1));
            this.modelVariablesList(end + 1) = startTime;
            
            stopTime = ModelParameter('StopTime', '');
            stopTime.value = num2str(this.t(length(this.t)));
            this.modelVariablesList(end + 1) = stopTime;
            
            fInputValues = ModelParameter('InputValues', '/D/F');
            fInputValues.value = mat2str(this.t);
            this.modelVariablesList(end + 1) = fInputValues;
            
            fTable = ModelParameter('Table', '/D/F');
            fTable.value = mat2str(this.F);
            this.modelVariablesList(end + 1) = fTable;
            
            vInputValues = ModelParameter('InputValues', '/D/V');
            vInputValues.value = mat2str(this.t);
            this.modelVariablesList(end + 1) = vInputValues;
            
            vTable = ModelParameter('Table', '/D/V');
            vTable.value = mat2str(this.V);
            this.modelVariablesList(end + 1) = vTable;
    
            mumax = ModelParameter('Value', '/Kinetics/Mmax');
            mumax.value = mat2str(this.Mmax);
            this.modelVariablesList(end + 1) = mumax;
    
            ks = ModelParameter('Value', '/Kinetics/Ks');
            ks.value = mat2str(this.Ks);
            this.modelVariablesList(end + 1) = ks;
        
            sInitialCondition = ModelParameter('InitialCondition', '/S/S');
            sInitialCondition.value = num2str(this.S0);
            this.modelVariablesList(end + 1) = sInitialCondition;
    
            k1 = ModelParameter('Value', '/S/K1');
            k1.value = mat2str(this.K1);
            this.modelVariablesList(end + 1) = k1;
            
            sin = ModelParameter('Value', '/S/Sin');
            sin.value = num2str(this.So);
            this.modelVariablesList(end + 1) = sin;
    
            xInitialCondition = ModelParameter('InitialCondition', '/X/X');
            xInitialCondition.value = num2str(this.X0);
            this.modelVariablesList(end + 1) = xInitialCondition;
        end 
        
        %% set values for the problem parameters before simulation
        function this = setValuesForTheProblemParameters(this, solutions)
            
            this.Mmax = solutions(:, 1);
            this.Ks = solutions(:, 2);
            this.K1 = solutions(:, 3);
        end
    
        %% interpolation of the results
        function [Xmod, Smod, Xexp, Sexp] = interpolate(this, output, solutionSize)
            
            Xout = output.Xout;
            Sout = output.Sout;
            T = output.tout;

            Xmod = interp1(T, Xout, this.t);
            Smod = interp1(T, Sout, this.t);

            Xexp = rep(this.Xd, [1 solutionSize]);
            Sexp = rep(this.Sd, [1 solutionSize]);
        end
    end

    methods (Access = public)
               
        function error = evaluate(this, solution)
            % start the simulation of the model for the current population
            [this, output] = this.execute(solution);
            
            [Xmod, Smod, Xexp, Sexp] = this.interpolate(output, size(solution));

            Ex = (Xmod - Xexp); 
            Es = (Smod - Sexp); 

            error = sum(Ex.*Ex) + sum(Es.*Es);
            
%             if ~this.dataPlotted
                EColiFBFPPlot.displayData(output.tout, output.Xout, output.Sout, this.t, this.Xd, this.Sd);
%                 this.dataPlotted = true;
%             else
%                 refreshdata
%                 drawnow update
%             end
        end
        
        %% additional processing of the result
        function [this, bestSolution, minError, fileName] = process(this, result)

            % find the min error and the corresponding estimations for
            % the problem parameters
            
            if isempty(result)
                exception = MException('InputParameters: EmptyList', 'The result is empty!');
                throw(exception);
            end

            % the estimations of the errors are stored in the first
            % column
            [minError, minIndex] = min(result(:, 1));

            bestSolution = result(minIndex, :);            
            %meanError = mean(result(:, 1));
                   
            % start simulation for the problem parameters' values
            % corresponding to the min error     
            this = this.setFirstStageParameterEstimations(bestSolution);
            
            % TODO: move these operations to a different class,
            % change the method to a non-static
            [this, output] = this.execute(this.estimations);

            Xmod = output.Xout;
            Smod = output.Sout;
            Tmod = output.tout;
            
            Xexp = this.Xd;
            Sexp = this.Sd;
            Texp = this.t;
                        
            % first stage of storing the data...
            fileName = EColiFBFPSaver.save(this, Tmod, Xmod, Smod);

            % TODO: use the data for plotting
            EColiFBFPPlot.displayDataOnNamedPlot(Tmod, Xmod, Smod, Texp, Xexp, Sexp, "Best solution");
        end
        
        %% set the estimations of the parameters 
        % evaluated during the first stage
        % the method might be overriden in the subclasses
        function this = setFirstStageParameterEstimations(this, bestSolution)
            if ~isempty(bestSolution)
                this.estimations = bestSolution(3: length(bestSolution));
            
                % TODO: clean, moved to the set function of estimations
                this.Mmax = this.estimations(1);
                this.Ks = this.estimations(2);
                this.K1 = this.estimations(3);
            end
        end
    end
    
    methods (Access = public, Static)
        %% generate char array with the results
        function solutionString = resultToString(bestSolution, minError)
            
            muMax = bestSolution(3);
            ks = bestSolution(4);
            k1 = bestSolution(5);
            
            solutionString = ['J = ' num2str(minError, '%.2f') ...
                    ' Mumax = ' num2str(muMax, '%.2f') ...
                    ' Ks = ' num2str(ks, '%.2f') ...
                    ' 1/Yxs = ' num2str(k1, '%.2f')];
            
        end
        
        %% the text shows that this is the basic model
        function description = toString()
            description = 'B';
        end
    end
end

