classdef EColiFBFPPlot
    % EColiFBFPPlot
    %   static methods for plotting the results of the simulation
    
    properties (Access = private, Constant)
    end
    
    methods (Access = public, Static)
        
        function handle = displayExpData(Texp, Xexp, Sexp)
            handle = figure;
              
            axes('FontSize', 14);

            subplot(2, 1, 1);
                                 
            plot(Texp, Xexp, 'k', 'LineWidth', 1, 'DisplayName', 'Experimental data'), hold on
                            
            xlabel('Time, [h]');
            ylabel('Biomass, [g.L^{-1}]');
            legend show
            legend('Location','northwest');
                       
            grid on;
            
            subplot(2, 1, 2);
                        
            plot(Texp, Sexp, 'k', 'LineWidth', 1, 'DisplayName', 'Experimental data'), hold on
            
            xlabel('Time, [h]');
            ylabel('Substrate, [g.L^{-1}]');      
            legend show
            
            axis([6 12 0 1.5]);
            grid on;
        end
        
        %% display model data on the same figure
        function handle = displayModelData(Tmod, Xmod, Smod, algName, handle, index)
            figure(handle);
            
            C = {'r', 'b', 'g', 'c', 'm', [.5 .6 .7], [.8 .2 .6]};
              
            axes('FontSize', 14);
           
            subplot(2, 1, 1);
                                 
            plot(Tmod, Xmod, 'color', C{index}, 'LineWidth', 1, 'DisplayName', algName), hold on
                            
            xlabel('Time, [h]');
            ylabel('Biomass, [g.L^{-1}]');
            legend show;
            legend('Location','northwest');                     
            
            grid on;
            drawnow update
            
            subplot(2, 1, 2);
                        
            plot(Tmod, Smod, 'color', C{index}, 'LineWidth', 1, 'DisplayName', algName), hold on
            
            xlabel('Time, [h]');
            ylabel('Substrate, [g.L^{-1}]');      
            legend show
            
            axis([6 12 0 1.5]);
            grid on;
            drawnow update
        end
        
        %% graphical comparison between the experimental and model data
        function displayData(Tmod, Xmod, Smod, Texp, Xexp, Sexp)
                                              
            figure (1)
              
            axes('FontSize', 14);
           
            subplot(2, 1, 1);
            
            plot(Texp, Xexp, 'r', 'LineWidth', 1), hold on
            plot(Tmod, Xmod, 'b', 'LineWidth', 1), hold on
                 
            xlabel('Time, [h]');
            ylabel('Biomass, [g.L^{-1}]');
            legend('Experimental data', 'Model', 'Location', 'northwest');
            grid on;
            drawnow update
                                   
            subplot(2, 1, 2);

            plot(Texp, Sexp, 'r', 'LineWidth', 1), hold on
            plot(Tmod, Smod, 'b', 'LineWidth', 1), hold on
            
            xlabel('Time, [h]');
            ylabel('Substrate, [g.L^{-1}]'); 
            grid on;
            legend('Experimental data', 'Model');
                         
            axis([6 12 0 1.5])
            drawnow update
        end
        
        %% graphical comparison between the experimental and model data
        % for the best estimates of the problem parameters
        function displayDataOnNamedPlot(Tmod, Xmod, Smod, Texp, Xexp, Sexp, name)
            
            handle = figure();
            handle.Name = name;
             
            axes('FontSize', 14);
           
            subplot(2, 1, 1);
                              
            plot(Texp, Xexp, 'r', 'LineWidth', 1), hold on
            plot(Tmod, Xmod, 'b', 'LineWidth', 1), hold on
            
            if name ~= ""
                title(name);
            end 
            
            xlabel('Time, [h]');
            ylabel('Biomass, [g.L^{-1}]');
            legend('Experimental data', 'Model', 'Location', 'northwest');
            grid on;
 %             axis([6 12 1 9])
                        
            subplot(2, 1, 2);
                        
            plot(Texp, Sexp, 'r', 'LineWidth', 1), hold on
            plot(Tmod, Smod, 'b', 'LineWidth', 1), hold on
            
            xlabel('Time, [h]');
            ylabel('Substrate, [g.L^{-1}]');      
            legend('Experimental data', 'Model'); 
            grid on;
            axis([6 12 0 1])
            
        end
        
        %% dissolved oxygen 
        function displayO2Data(DOmod, Tmod, DOd, tdo1)

            Texp = Tmod(1) : 0.3 : Tmod(length(Tmod));
            Texp = [Texp Tmod(length(Tmod))];
            
            DOexp = interp1(tdo1, DOd, Texp);
            
            figure (3)
                      
            plot(Texp, DOexp, 'r*'), 
            grid on,
            hold on, 
%             title('Cultivation of E. coli MC4110'), 
            xlabel('Time, [h]'), ylabel('Dissolved oxygen, [%]');
            
            plot(Tmod, DOmod, 'b' ), 
            
            legend('Experimental data', 'Model data')           
            drawnow update
        end
        
        %% plot the oxygen data for the best estimates of the parameters
        function displayO2DataOnNamedFigure(DOmod, Tmod, DOd, tdo1, name)
            
            Texp = Tmod(1) : 0.3 : Tmod(length(Tmod));
            Texp = [Texp Tmod(length(Tmod))];
            
            DOexp = interp1(tdo1, DOd, Texp);
            
            if (name == "")
                figure (3)
            else
                handle = figure();
                handle.Name = name;
            end
            
            plot(Texp, DOexp, 'r*'), 
            grid on,
            hold on,
            xlabel('Time, [h]'), ylabel('Dissolved oxygen, [%]');
            
            if name ~= ""
                title(name);
            end
            
            plot(Tmod, DOmod, 'b' ), 
            legend('Experimental data', 'Model data')
            
            drawnow update
        end
    end
    
    methods (Access = private, Static)
        %% plot the experimental data
        function plotBiomassAndSubstrate(X, S, T)
            figure ('Name', 'Comparison');
            
            axes('FontSize', 14);
           
            subplot(2, 1, 1);
            
            title('Biomass');
            
            xlabel('Time, [h]');
            ylabel('Biomass, [g.L^{-1}]');
            
            grid on;
            plot(T, X, 'r', 'DisplayName', 'Experimental data', 'LineWidth', 0.5);
            hold on
            
            linkdata on
            
            axis([6 12 1 9])
            
            legend;
            drawnow update
            
            subplot(2, 1, 2);
            
            title('Substrate');
            
            xlabel('Time, [h]');
            ylabel('Substrate, [g.L^{-1}]');
            
            grid on;
            plot(T, S, 'r', 'DisplayName', 'Experimental data', 'LineWidth', 0.5); 
            hold on
            
            linkdata on
            
            axis([6 12 0 1])
            
            legend;
            drawnow update
            
            this.experimentalDataPlotted = true;
        end
    end
    
end

