classdef GABFPModel < ProcessModel
    % GAFBFermentationProcess describes the basic model
    %   with the basic list of model parameters
    
    properties
        % model parameters
        t, Xd, Sd, GAd, O2d, X0, S0, GA0, O20, Mumax, K, Muga, Kga, K1, K2, K3

        % indicator for plotting the data for the first time
        dataPlotted  logical = false;
        
        % estimations of the parameters
        estimations = [];
    end
    
    methods %(Access = public)
        %% constructor
        function this = GABFPModel(modelName, path, selectedEquationsList,...
                                       selectedKinetics, pathToExperimentalData)
            if nargin ~= 5
                 exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                 throw(exception);
            end
            
            this = this@ProcessModel(modelName, path, selectedEquationsList, selectedKinetics);
            
            % set the number of variables and their ranges
            this = this.initializeProblemParameters();
            this.nvar = length(this.problemParametersList);
            
            this.pathToExperimentalData = pathToExperimentalData;
        end
        
        %% set the estimations of the problem parameters
        function this = set.estimations(this, problemEstimations)
            if ~isempty(problemEstimations)
                this.estimations = problemEstimations;

                this.Mumax  = this.estimations(1);
                this.K      = this.estimations(2);
                this.Muga   = this.estimations(3);
                this.Kga    = this.estimations(4);
                this.K1     = this.estimations(5);
                this.K2     = this.estimations(6);
                this.K3     = this.estimations(7);

            end
        end
    end
    
    methods (Access = protected) 
        %% initialize the problem parameters' list
        function this = initializeProblemParameters(this)
            
            Mumax = Parameter("Mumax", 0.186, 0.15, 0.2);
            this.problemParametersList(end + 1) = Mumax;
            
            K = Parameter("K", 8.031, 6.50, 9);
            this.problemParametersList(end + 1) = K;
            
            Muga = Parameter("Muga", 0.335, 0.3, 0.4);
            this.problemParametersList(end + 1) = Muga;

            Kga = Parameter("Kga", 107.064, 90, 120);
            this.problemParametersList(end + 1) = Kga;
 
            K1 = Parameter("1/Yxs", 6.262, 5, 7);
            this.problemParametersList(end + 1) = K1;
 
            K2 = Parameter("1/Ysga", 0.028, 0.02, 0.03);
            this.problemParametersList(end + 1) = K2;

            K3 = Parameter("1/Yox", 0.81, 0.70, 0.90);
            this.problemParametersList(end + 1) = K3;
            
        end
    end
    
    % virtual methods, to be overloaded in the subclasses
    % loading data from the experimental data files
    % executing the process
    methods (Access = protected)
    
        function [this, data] = loadData(this) 
            
            % TODO: some validation of the input data
            data = GluconicAcidBFPReader.loadExperimentalData(this.pathToExperimentalData);
                      
            this.t      = data(:, 1);
            this.Xd     = data(:, 2);
            this.Sd     = data(:, 3); 
            this.GAd    = data(:, 4);
            this.O2d    = data(:, 5);
          
            this.X0     = this.Xd(1);
            this.S0     = this.Sd(1);
	        this.GA0    = this.GAd(1);
            this.O20    = this.O2d(1);

        end
        
         %% set the initial parameters (model and block parameters)
        function this = initializeModelParametersList(this)
            
            startTime = ModelParameter('StartTime', '');
            startTime.value = num2str(this.t(1));
            this.modelVariablesList(end + 1) = startTime;
            
            stopTime = ModelParameter('StopTime', '');
            stopTime.value = num2str(this.t(length(this.t)));
            this.modelVariablesList(end + 1) = stopTime;
            
        
            Mumax = ModelParameter('Value', '/Rx/Mumax');
            Mumax.value = mat2str(this.Mumax);
            this.modelVariablesList(end + 1) = Mumax;
            
            K = ModelParameter('Value', '/Rx/K');
            K.value = mat2str(this.K);
            this.modelVariablesList(end + 1) = K;
    
            xInitialCondition = ModelParameter('InitialCondition', '/X');
            xInitialCondition.value = num2str(this.X0);
            this.modelVariablesList(end + 1) = xInitialCondition;
        
                       
            Muga = ModelParameter('Value', '/Rga/Muga');
            Muga.value = mat2str(this.Muga);
            this.modelVariablesList(end + 1) = Muga;
            
            Kga = ModelParameter('Value', '/Rga/Kga');
            Kga.value = mat2str(this.Kga);
            this.modelVariablesList(end + 1) = Kga;
    
            xInitialCondition = ModelParameter('InitialCondition', '/GA');
            xInitialCondition.value = num2str(this.GA0);
            this.modelVariablesList(end + 1) = xInitialCondition;
            
            
            K1 = ModelParameter('Value', '/S/K1');
            K1.value = mat2str(this.K1);
            this.modelVariablesList(end + 1) = K1;
            
            K2 = ModelParameter('Value', '/S/K2');
            K2.value = mat2str(this.K2);
            this.modelVariablesList(end + 1) = K2;
           
            sInitialCondition = ModelParameter('InitialCondition', '/S/S');
            sInitialCondition.value = num2str(this.S0);
            this.modelVariablesList(end + 1) = sInitialCondition;
    
            K3 = ModelParameter('Value', '/O2/K3');
            K3.value = mat2str(this.K3);
            this.modelVariablesList(end + 1) = K3;
            
%             Kla = ModelParameter('Value', '/O2/Kla');
%             Kla.value = num2str(this.Kla);
%             this.modelVariablesList(end + 1) = Kla;
%             
%             O2* = ModelParameter('Value', '/O2/O2*');
%             O2*.value = num2str(this.O2*);
%             this.modelVariablesList(end + 1) = O2*; 
            
            sInitialCondition = ModelParameter('InitialCondition', '/O2/O2int');
            sInitialCondition.value = num2str(this.O20);
            this.modelVariablesList(end + 1) = sInitialCondition;
           
        end 
        
        %% set values for the problem parameters before simulation
        function this = setValuesForTheProblemParameters(this, solutions)
            
            this.Mumax  = solutions(:, 1);
            this.K      = solutions(:, 2);
            this.Muga   = solutions(:, 3);
            this.Kga    = solutions(:, 4);
            this.K1     = solutions(:, 5);
            this.K2     = solutions(:, 6);
            this.K3     = solutions(:, 7);
        end
    
        %% interpolation of the results
        function [Xmod, Smod, GAmod, O2mod, Xexp, Sexp, GAexp, O2exp] = interpolate(this, output, solutionSize)
            
            Xout    = output.Xout;
            Sout    = output.Sout;
            GAout   = output.GAout;
            O2out   = output.O2out;
            T       = output.tout;

            Xmod    = interp1(T, Xout, this.t);
            Smod    = interp1(T, Sout, this.t);
            GAmod   = interp1(T, GAout, this.t);
            O2mod   = interp1(T, O2out, this.t);

            Xexp    = rep(this.Xd, [1 solutionSize]);
            Sexp    = rep(this.Sd, [1 solutionSize]);
            GAexp   = rep(this.GAd, [1 solutionSize]);
            O2exp   = rep(this.O2d, [1 solutionSize]);
        end
    end

    methods (Access = public)
               
        function error = evaluate(this, solution)
            % start the simulation of the model for the current population
            [this, output] = this.execute(solution);
            
            [Xmod, Smod, GAmod, O2mod, Xexp, Sexp, GAexp, O2exp] = this.interpolate(output, size(solution));

            Ex = (Xmod - Xexp); 
            Es = (Smod - Sexp); 
            Ega = (GAmod - GAexp); 
            Eo2 = (O2mod - O2exp); 

            error = sum(Ex.*Ex) + sum(Es.*Es) + sum(Ega.*Ega) + sum(Eo2.*Eo2);
            
            if ~this.dataPlotted
                GABFPPlot.displayData(output.tout, output.Xout, output.Sout, output.GAout, output.O2out, this.t, this.Xd, this.Sd, this.GAd, this.O2d);
                this.dataPlotted = true;
            else
                refreshdata
                drawnow
            end
        end
        
        %% additional processing of the result
        function [this, bestSolution, meanError, fileName] = process(this, result)

            % find the min error and the corresponding estimations for
            % the problem parameters

            % the estimations of the errors are stored in the first
            % column
            bestSolution = [];
            
            if ~isempty(result)
                [~, minIndex] = min(result(:, 1));

                bestSolution = result(minIndex, :);            
                meanError = mean(result(:, 1));
            end

            % start simulation for the problem parameters' values
            % corresponding to the min error     
            
            this = this.setFirstStageParameterEstimations(bestSolution);
            
            % TODO: move these operations to a different class,
            % change the method to a non-static
            [this, output] = this.execute(this.estimations);

            Tmod = output.tout;
            Xmod = output.Xout;
            Smod = output.Sout;
            GAmod = output.GAout;
            O2mod = output.O2out;
            
            % first stage of storing the data...
            fileName = GluconicAcidBFPSaver.save(this, Tmod, Xmod, Smod, GAmod, O2mod);

            % TODO: use the data for plotting
            GABFPPlot.displayDataOnNamedPlot(Tmod, Xmod, Smod, GAmod, O2mod, this.t, this.Xd, this.Sd, this.GAd, this.O2d, "Best solution");
        end
        
        %% set the estimations of the parameters 
        % evaluated during the first stage
        % the method might be overriden in the subclasses
        function this = setFirstStageParameterEstimations(this, bestSolution)
            if ~isempty(bestSolution)
                this.estimations = bestSolution(3: length(bestSolution));
            
                % TODO: clean, moved to the set function of estimations
                this.Mumax = this.estimations(1);
                this.K = this.estimations(2);
                this.Muga = this.estimations(3);
                this.Kga = this.estimations(4);
                this.K1 = this.estimations(5);
                this.K2 = this.estimations(6);
                this.K3 = this.estimations(7);
            end
        end
    end
    
    methods (Access = public, Static)
        %% generate char array with the results
        function solutionString = resultToString(bestSolution, meanError)
            
            Mumax = bestSolution(3);
            K = bestSolution(4);
            Muga = bestSolution(5);
            Kga = bestSolution(6);
            K1 = bestSolution(7);
            K2 = bestSolution(8);
            K3 = bestSolution(9);
            
            solutionString = ['J = ' num2str(meanError, '%.2f') ...
                    ' Mumax = ' num2str(Mumax, '%.2f') ...
                    ' K = ' num2str(K, '%.2f') ...
                    ' Muga = ' num2str(Muga, '%.2f') ...
                    ' Kga = ' num2str(Kga, '%.2f') ...
                    ' K1 = ' num2str(K1, '%.2f') ...
                    ' K2 = ' num2str(K2, '%.2f') ...
                    ' K3 = ' num2str(K3, '%.2f')];
        end
        
        function description = toString()
            description = '';
        end
    end
end

