classdef (Abstract) ProcessModel < matlab.mixin.Heterogeneous
    % ProcessModel class describes the model of a certain process
    %   Characteristics:
    %       modelName (char array)
    %       path (string)
    %       modelParametersList 
    %       equationDescriptionsList
    %       kinetics (string)
    %
    %   the number of variables and their raanges are to be extracted from
    %   the properties of the process
    %
    %       nvar (number of variables)
    %       ranges (matrix, nvar x 2 with the ranges for the variables)
    %
    %   pathToExperimentalData (string)
    %
    %   how many exutions will be made in order to evaluate the model?
    %   the basic model is executed in one step
    %   more complex models are executed in two steps
    %       executesInOneStep (logical), default value: 1
    
    properties
        modelName char %{ mustBeNonempty }
        path char %{ mustBeNonempty } 
        
        modelVariablesList = ModelParameter.empty()
        
        equationDescriptionsList EquationsList
        kinetics string %{ mustBeNonempty } 
        
        nvar double { mustBeReal, mustBeNonnegative }
        problemParametersList = Parameter.empty()
        
        pathToExperimentalData string { mustBeNonempty } = pwd % mustBeFile
        
        executesInOneStep logical = 1
        
        % input of type SimulationInput
        simulationInput
    end
    
    methods
        %% Constructor
        % the specified Simulink model is loaded into the memory
        function this = ProcessModel(modelName, path, selectedEquationsList, selectedKinetics)
            if nargin ~= 4
                 exception = MException('InputParameters:WrongNumber', 'Wrong number of input paramters!');
                 throw(exception);
            end
            
            this.modelName = modelName;
            this.path = path;
            
            this.equationDescriptionsList = selectedEquationsList;
            this.kinetics = selectedKinetics;
            
            this.loadModel();
        end
        
        %% Destructor
        % the model should be closed when the object is destructed
        function delete(this)
            pathToModel = ['.\Models\' this.path '\' this.modelName];
            close_system(pathToModel);
        end
        
        %% set model, check if the model name is empty
        function this = set.modelName(this, newModelName)
            if isempty(newModelName)
                exception = MException('InputParameters:EmptyString', 'The model name should not be empty!');
                throw(exception);
            end
            
            this.modelName = newModelName;
        end
        
        %% set path, check if the path string is empty
        function this = set.path(this, newPath)
            if isempty(newPath)
                exception = MException('InputParameters:EmptyString', 'The path string should not be empty!');
                throw(exception);
            end
            
            this.path = newPath;
        end
        
        %% set kinetics, check if the input string is empty
        function this = set.kinetics(this, newKinetics)
            if newKinetics == ""
                exception = MException('InputParameters:EmptyString', 'The kinetics should not be empty!');
                throw(exception);
            end
            
            this.kinetics = newKinetics;
        end
    end
    
    % virtual methods, to be overriden in the subclasses
    methods (Abstract, Access = protected)
        % loading data from the experimental data files
        % executing the process
        [this, data] = loadData(this)
        
        % set values for the problem parameters
        this = setValuesForTheProblemParameters(this, solution)
        
        % initialize model parameters
        this = initializeModelParametersList(this)
    end
    
    methods (Access = private)
        %% load model into the memory
        function this = loadModel(this)
            pathToModel = ['.\Models\' this.path '\' this.modelName];
            % TODO: the model should be closed later... 
            % when? in the destructor?
            load_system(pathToModel);
        end
        
        %% set values of the model parameters before simulation
        function this = setModelParametersBeforeSimulation(this, solution)
            this = this.loadData();
            
            this = this.setValuesForTheProblemParameters(solution);
            this = this.initializeModelParametersList();
            
            set_param(this.modelName, 'Solver', 'ode45', 'RelTol', '1e-6', 'AbsTol', '1e-6', 'MaxStep', '0.001'); 

            this.simulationInput = Simulink.SimulationInput(this.modelName);
            
            % Set simulation parameters
            numberOfParameters = length(this.modelVariablesList);
            
            for index = 1: numberOfParameters
                currentParameter = this.modelVariablesList(index);
                
                if ~currentParameter.isBlockParameter()
                    this.simulationInput = this.simulationInput.setModelParameter(currentParameter.name, currentParameter.value);
                else
                    this.simulationInput = this.simulationInput.setBlockParameter([this.modelName currentParameter.blockPath], currentParameter.name, currentParameter.value);
                end
            end
        end
    end 
    
    methods (Access = protected)
        %% load and execute the model in Simulink       
        function [this, output] = execute(this, solution)
            % TODO: revise! some of the parameters should be loaded only
            % once; only the problem parameters are different on each run
            this = this.setModelParametersBeforeSimulation(solution);
            % SimulationOutput object
            output = sim(this.simulationInput);
        end            
    end
    
    methods (Abstract, Access = public)
        
        % evaluate the current solution, using the experimental data
        error = evaluate(this, solution)
        
        % set the estimations of the parameters 
        % evaluated during the first stage
        % the method might be overriden in the subclasses
        this = setFirstStageParameterEstimations(this, bestSolution)
    end
    
    
    methods (Access = public, Static = true)
        %% check if all the equations in the input list are compulsory
        function onlyCompulsoryEquations = onlyCompulsoryEquationsAreSelected(listOfSelectedEquations)
            onlyCompulsoryEquations = 1;
            
            numberOfEquations = length(listOfSelectedEquations);
            
            index = 1;
            while index <= numberOfEquations && onlyCompulsoryEquations
                if not(listOfSelectedEquations.getElementAt(index).isCompulsory)
                    onlyCompulsoryEquations = 0;
                end
                index = index + 1;
            end
        end
        
        %% find the indices of the additional equations in the input list
        function listOfAdditionalEquations = findAdditionalEquations(listOfSelectedEquations)
            listOfAdditionalEquations = EquationDescription.empty;
            
            numberOfEquations = length(listOfSelectedEquations);
            
            for index = 1: numberOfEquations
                currentEquation = listOfSelectedEquations(index);
               
                if ~currentEquation.isCompulsory
                    listOfAdditionalEquations(end + 1) = currentEquation;
                end
            end
        end 
    end
end

