classdef EvolutionaryAlgorithm < OptimizationAlgorithm
    %   Properties:
    %       maxIter (the algrithms stops if the solution has not been
    %       improved for maxIter steps)
    %
    %       sigma
    
    properties
        maxIter     int16     { mustBeNonnegative } = 2;
        step        double    { mustBeReal, mustBeNonnegative } = 0.5;
    end
    
    methods (Access = public)
        %% constructor
         function this = EvolutionaryAlgorithm(description, problem)
            if nargin ~= 2
                exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                throw(exception);           
            end
            
            this = this@OptimizationAlgorithm(description, problem);
            
            this = this.setParameters();
         end
         
        % set values for the algorithm parameters,
        % the values are in the description object of the algorithm,
        % extracted from the corresponding edit fields in the ui
        function this = setParameters(this)
            
            parametersList = this.description.parametersList;
            this.maxIter = parametersList(1).value;
            this.step = parametersList(2).value;
          
        end
         
        %% overriding the abstract methods
        function [this, solution] = generateInitialPopulation(this)
            
            solution = this.description.initialPopulation;  
            
            % for the first step of the extended model, REDUNDANT
%             if size(solution, 2) ~= this.problem.model.nvar
%                 solution = solution(:, 1: this.problem.model.nvar);
%             end
            
        end
          
        function error = evaluatePopulation(this, population)
            error = this.problem.model.evaluate(population);
        end
          
        function nextPopulation = generateNextPopulation(this, population, ~)
            [~, numberOfParameters] = size(population);
            randomvec = ones(numberOfParameters, 1) + (this.step/100) * randn(numberOfParameters, 1);
            nextPopulation = population.*randomvec';
        end
    end
    
    %% override protected abstract methods
    methods (Access = protected)
        function result = executeModel(this)
            disp('START ');
            
            % generate the initial solution
            [this, solution] = this.generateInitialPopulation();
          
            bestSolution = solution;  
            smallestError = 1000000;
            
            i = 0; 
            I = 0; 

            % the min error, estimation time and the optimal parameter
            % estimations are stored at each run
            
            result = [];
            % [~, numberOfParameters] = size(solution);
            % result = zeros(2 + numberOfParameters, this.maxIter);
            
            plotError = smallestError * ones(1, this.maxIter);
                        
            while ((i - I) < this.maxIter) 
                
                startTime = cputime;
                
                i = i + 1;
                currentError = this.evaluatePopulation(solution);
                              
                plotError(i) = currentError;
                
                 if currentError < smallestError
                    bestSolution = solution;
                    smallestError = currentError;
                 end
                
                 solution = generateNextPopulation(this, bestSolution);
                 [~, I] = min(plotError);
        
                 estimationTime = cputime - startTime;
        
                 result_current_iteration = [smallestError estimationTime bestSolution];
                 result = [result; result_current_iteration]
                 % result(i, :) = result_current_iteration;

            end
            
            % ToDo: to remove the last maxIter records (they should be the
            % same?)
        end
    end
    
    
    %% to return the number of parameters
    methods (Static, Access = public)
        function numberOfParameters = getNumberOfParameters()
            numberOfParameters = 2;
        end
    end
end

