classdef GeneticAlgorithm < OptimizationAlgorithm
    %   Properties:
    %       mutr (mutation rate)
    %       xovr (crossover rate)
    %       nind (number of individuals)
    %       maxgen (Max number of generations)
    %       ggap (Generation gap, how many new individuals are created)
    %       nrun (Number of runs)
    %
    %       fildDR (field description matrix, size: numberOfParameters x 2)

    %% properties of the subclass
    properties
        mutr    double     { mustBeReal, mustBeNonnegative } = 0.05;
        xovr    double     { mustBeReal, mustBeNonnegative } = 0.7;
        nind    int16      { mustBeNonnegative } = 20;
        maxgen  int16      { mustBeNonnegative } = 20;
        ggap    double     { mustBeReal, mustBeNonnegative } = 0.97;
        nrun    int16      { mustBeNonnegative } = 1;
        
        fieldDR 
    end
    
    %% constants of the subclass
    properties (Constant)
        % Selection function
        SEL_F = 'rws';	
        
        % Recombination fun.
        XOV_F = 'recint';
        
        % Mutation function
        MUT_F = 'mutbga';	
    end
    
    %% private methods, accessible only inside the class
    methods (Access = private)
        
        % Build field description matrix
        function this = buildFieldMatrix(this)
            
            problemVariablesList = this.problem.model.problemParametersList;
            numberOfVariables = this.problem.model.nvar;

            this.fieldDR = zeros(2, numberOfVariables);

            for index = 1: numberOfVariables
                this.fieldDR(:, index) = [problemVariablesList(index).lowerBound; problemVariablesList(index).upperBound];
            end
        end
    
    end
   
    methods (Access = public)
        %% constructor, initialization
        function this = GeneticAlgorithm(description, problem)
            if nargin ~= 2
                exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                throw(exception);           
            end
            
            this = this@OptimizationAlgorithm(description, problem);
           
            this = this.setParameters();
            this = this.buildFieldMatrix();
        end
        
        % set values for the algorithm parameters,
        % the values are in the description object of the algorithm,
        % extracted from the corresponding edit fields in the ui
        function this = setParameters(this)
            
            this.mutr   = this.description.parametersList(1).value;
            this.xovr   = this.description.parametersList(2).value;
            this.nind   = this.description.parametersList(3).value;
            this.maxgen = this.description.parametersList(4).value;
            this.ggap   = this.description.parametersList(5).value;
            this.nrun   = this.description.parametersList(6).value;
            
        end
    
        %% overriding the abstract methods

        % generate the initial population
        % required parameters: 
        %   number of variables, 
        %   ranges for the valid values of the variables
        function [this, population, evaluations] = generateInitialPopulation(this)

        % for the first step of the extended model, REDUNDANT
%             if size(this.fieldDR, 2) ~= this.problem.model.nvar
%                 this.fieldDR = this.fieldDR(:, 1: this.problem.model.nvar);
%             end

            % generate the initial population
            population = crtrp(this.nind, this.fieldDR);

            % Evaluate the initial population
            evaluations = this.evaluatePopulation(population)';
        end

        % evaluate the current population
        function error = evaluatePopulation(this, population)
            error = this.problem.model.evaluate(population);
        end

        % generate the next population
        function [nextPopulation, evaluations] = generateNextPopulation(this, population, evaluations)
            
            % Fitness assignement of the entire population
            rankedFitnessValues = ranking(evaluations);

            % Select individuals from population
            selectedIndividuals = select(this.SEL_F, population, rankedFitnessValues, this.ggap);

            % Recombine selected individuals (crossover)
            selectedIndividuals = recombin(this.XOV_F, selectedIndividuals, this.xovr);

            % Mutate offspring
            selectedIndividuals = mutbga(selectedIndividuals, this.fieldDR, this.mutr);

            % Evaluate the newly generated population
            evaluationsOfSelectedIndiviaduals = this.evaluatePopulation(selectedIndividuals);

            % Insert offspring in population replacing parents
            [nextPopulation, evaluations] = reins(population, selectedIndividuals, 1, [1 0.8], evaluations, evaluationsOfSelectedIndiviaduals');
        end
            
        % generate char array with the description of the optimization algorithm and the problem
        function description = toString(this)
            
            description = [char(this.problem.description.name) ', ' ...
                    char(this.problem.model.kinetics) ', ' ...
                    char(this.description.shortName)];
            
        end
        
        % generate full description of the algorithm
        function fullDescription = generateFullDescription(this)
            fullDescription = ...
                this.description.shortName + ...
                " MUTR = " + num2str(this.mutr) + ...
                " XOVR = " + num2str(this.xovr) + ...
                " NIND = " + num2str(this.nind) + ...
                " MAXGEN = " + num2str(this.maxgen) + ...
                " GGAP = " + num2str(this.ggap) + ...
                " NRUN = " + num2str(this.nrun);
        end
    end
    
    %% override protected abstract methods
    methods ( Access = protected)
        % execute a certain number of iterations...
        function result = executeModel(this)

            disp('START GA');
            
            % generate the initial population
            [this, population, evaluations] = this.generateInitialPopulation();

            % the min error, estimation time and the optimal parameter
            % estimations are stored on each run
            result = zeros(this.nrun, this.problem.model.nvar + 2);

            % the algorithm should be started a certain number of times
            for run = 1: this.nrun

                startTime = cputime; 

                % array of the min errors for each generation
                minErrors = zeros(this.maxgen, 1);
                % param set for each generation
                params = zeros(this.maxgen, this.problem.model.nvar);
                
                % iterate through the generation
                for gen = 1: this.maxgen
                    [population, evaluations] = this.generateNextPopulation(population, evaluations);

                    [minErrorCurrentGen, i] = min(evaluations);
                    minErrors(gen) = minErrorCurrentGen;

                    bestParamsCurrGen = population(i, :);
                    params(gen, :) = bestParamsCurrGen;
                end

                [minError, min_index] = min(minErrors);
                optimalParameters = params(min_index, :);
                estimationTime = cputime - startTime;

                result(run, :) = [minError estimationTime optimalParameters]
            end
        end
    end
end

