classdef (Abstract) OptimizationAlgorithm < matlab.mixin.Heterogeneous
    % OptimizationAlgorithm is an abstract class which descrines the basic
    % behaviour expected of an optimization algorithm
    %
    %   Characteristics:
    %       description (AlgorithmDescription)
    %
    %       problem (FermentationProcess)
    %       
    %   the number of variables and their ranges are to be extracted from
    %   the properties of the process
    %
    %       nvar (number of variables)
    %       ranges (matrix, nvar x 2 with the ranges for the variables
    
    properties
        description AlgorithmDescription
        
        problem FermentationProcess
    end
    
    methods
        %% constructor
        % Input arguments: 
        %           description (AlgorithmDescription), 
        %           problem (ProcessModel)
        function this = OptimizationAlgorithm(description, problem)
            if nargin ~= 2
                exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                throw(exception);           
            end
            
            this.description = description;
            this.problem = problem;
        end
    end
    
    %% abstract methods, to be overriden in the subclasses
    % TODO: some of these methods should be in the protected section of the
    % base class
    methods (Abstract, Access = public)
        
        % generate the initial population
        % required parameters: 
        %   number of variables, 
        %   ranges for the valid values of the variables
        [this, population, evaluations] = generateInitialPopulation(this);
        
        % evaluate the current population
        error = evaluatePopulation(this, population);
        
        % generate the next population
        [nextPopulation, evaluations] = generateNextPopulation(this, population, evaluations);
        
    end
    
    %% no need to override the method
    methods (Access = public)
        % execute a certain number of iterations...
        function [this, result] = execute(this)
            % if the basic model is executed
            if this.problem.model.executesInOneStep
                result = this.executeModel();
            else
                % extract the basic model, 
                % the first stage is to execute the basic model
                % the evaluations of the paramters are to be used 
                % in the second stage (evaluation of the complex model)
                
                originalModel = this.problem.model;
                
                % TODO: define extractBasicModel in the extended model
                % class
                basicModel = originalModel.extractBasicModel();
                this.problem.model = basicModel;
                
                % the basic model should be executed here
                result = this.executeModel();
                [~, bestSolution, ~] = basicModel.process(result);
                
                originalModel = originalModel.setFirstStageParameterEstimations(bestSolution);
                this.problem.model = originalModel;
                % start the second stage
                result = this.executeModel();
            end
        end
        
        function description = toString(this)
            
            description = [char(this.problem.description.name) ', ' ...
                    char(this.problem.model.kinetics) ', ' ...
                    char(this.description.shortName)];
            
        end  
        
        % generate full description of the algorithm
        function fullDescription = generateFullDescription(this)
            fullDescription = "";
        end
    end
    
    %% abstract methods to be overriden in the subclasses
    methods (Abstract, Access = protected)
        
        % execute a certain number of iterations on a specified model
        % when the model is too complex the execution will be in two steps
        result = executeModel(this);
    end
end

