classdef Parameter
    % Parameter describes a single parameter of the fermentation
    % process model or a parameter of a optimization algorithm
    %   object characteristics: 
    %       name (string),
    %       default value (double), 
    %       upper and lower bounds of the parameter (doubles)
    
    properties 
        name string { mustBeNonempty } = "Undefined"
    end
    
    properties %(Dependent)
        lowerBound double { mustBeReal, mustBeNonnegative } = 0
        upperBound double { mustBeReal, mustBeNonnegative } = realmax
        
        defaultValue double { mustBeReal, mustBeNonnegative } = 0.0
        
        % check the value upon intialization with the LB and UB set for the
        % parameter
        value double { mustBeReal, mustBeNonnegative } = 0.0
    end
    
    methods (Access = private, Static)
        %% function to validate the range [lowerBound, upperBound]
        function isValid = isValidRange(lowerBound, upperBound)          
            isValid = (lowerBound < upperBound);
        end
    end
    
    methods 
         %% constructor of the class
         function this = Parameter(name, defaultValue, lowerBound, upperBound)
            % if there are any arguments
            if nargin ~= 4
                exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                throw(exception);
            end
             
            this.name = name;

            this.lowerBound = lowerBound;
            this.upperBound = upperBound;

            this.defaultValue = defaultValue; 
         end
         
         %% function to validate of the value of the prameter is within the specified range
        function isValid = isValueWithinTheRange(this, value)
            isValid = (value >= this.lowerBound && value <= this.upperBound);
        end
         
         %% public function to set the default value of the parameter
         function this = set.defaultValue(this, defaultValue)
            if ~this.isValueWithinTheRange(defaultValue)
                % in case of error, an exception is thrown to prevent 
                % constructing the object
                exception = MException('Parameter:WrongInput', 'The specified parameter value is not in the range [LB, UB]!');
                throw(exception);
            end
             
            this.defaultValue = defaultValue;
         end
         
         %% public function to set the lower bound of the parameter
         function this = set.lowerBound(this, lowerBound)
            if ~Parameter.isValidRange(lowerBound, this.upperBound) 
                 % in case of error, an exception is thrown to prevent 
                 % constructing the object
                 exception = MException('Parameter:WrongInput', 'The specified value for lower bound is bigger than the upper bound!');
                 throw(exception);
            end
             
            this.lowerBound = lowerBound;
         end
               
         %% public function to set the upper bound of the parameter
         function this = set.upperBound(this, upperBound)
            if ~Parameter.isValidRange(this.lowerBound, upperBound)
                % in case of error, an exception is thrown to prevent 
                % constructing the object
                exception = MException('Parameter:WrongInput', 'The specified value for upper bound is lower than the lower bound!');
                throw(exception);
            end
             
            this.upperBound = upperBound;
         end
         
         %% public function to set the current value of the parameter
         function this = set.value(this, value)
             if ~this.isValueWithinTheRange(value)  
                 % in case of error, an exception is thrown to prevent 
                 % constructing the object
                 exception = MException('Parameter:WrongValue', ['The specified value for ' char(this.name) ' is not in the range [LB, UB]!']);
                 throw(exception);
             end
             
             this.value = value;
         end
     end
end

