classdef AlgorithmDescription
    % Characteristics:
    %
    %   name (string)
    %   numberOfParameters (unsigned integer)
    %   parametersList    
    
    properties
         name       string { mustBeNonempty } = "Undefined"
         shortName  char = ''
         
         numberOfParameters uint16 { mustBeNonnegative } = 0;
         parametersList = Parameter.empty;
         
         hasRandomInitialPopulation logical = true;
         % when the initial population is not generated randomly,
         % it should be given by the user (ui)
         initialPopulation
    end
    
    methods
        %% constructor
        function this = AlgorithmDescription(name, shortName, listOfParameters)
            if nargin ~= 3
                exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                throw(exception);           
            end
            
            this.name = name;
            this.shortName = shortName;
            
            this.numberOfParameters = length(listOfParameters);
            this.parametersList = listOfParameters;
        end
        
        %% check if the description is empty
        function empty = isEmpty(this)
            empty = isempty(this.parametersList);
        end
        
        %% set the property for the initial population
        function this = set.hasRandomInitialPopulation(this, value)
            this.hasRandomInitialPopulation = value;
        end
        
        %% change the parameters list
        function this = set.parametersList(this, newList)
            this.parametersList = newList;
        end
        
        %% get the number of parameters
        function value = get.numberOfParameters(this)
            value = length(this.parametersList);
        end
    end
    
    methods (Access = public, Static = true)
        %% creates an object with the description of the GA
        % TODO: check the default values and the ranges of the parameters
        function ga = createGeneticAlgorithmDescription()
            
            %parametersList = Parameter.empty;
            
            mutr = Parameter("MUTR", 0.01, 0.001, 0.1);
            %parametersList(end + 1) = mutr;
            
            xovr = Parameter("XOVR", 0.7, 0.5, 1);
            %parametersList(2) = xovr;
            
            nind = Parameter("NIND", 10, 1, 50);
            %parametersList(3) = nind;
            
            maxgen = Parameter("MAXGEN", 2, 1, 50);
            %parametersList(4) = maxgen;

            ggap = Parameter("GGAP", 0.97, 0.5, 1);
            %parametersList(5) = ggap;
            
            nrun = Parameter("NRUN", 1, 1, 10);
            %parametersList(6) = nrun;
            
            parametersList = [mutr xovr nind maxgen ggap nrun];
            ga = AlgorithmDescription("Genetic Algorithm", 'GA', parametersList);
        end
        
        function ea = createEvolutionaryAlgorithmDescription()
                       
            maxIter = Parameter("Max Iter", 2, 1, 5);
            
            step = Parameter("Step", 0.5, 0, 5);
            parametersList = [maxIter step];
            
            ea = AlgorithmDescription("Evolutionary Algorithm", 'EA', parametersList);
            ea.hasRandomInitialPopulation = false;
        end
        
        function eaga = createEAGAHybridDescription()
            ea = AlgorithmDescription.createEvolutionaryAlgorithmDescription();
            ga = AlgorithmDescription.createGeneticAlgorithmDescription();
            
            parametersList = [ga.parametersList ea.parametersList];
            
            eaga = AlgorithmDescription("EA-GA Hybrid", 'EA-GA', parametersList);
            eaga.hasRandomInitialPopulation = false;
        end
    end
    
end

