classdef FermentationProcessDescription
    % FermentationProcess describes a single fermentation process
    %   object characteristics: 
    %
    %       name (string)
    %       description (string)
    %       
    %       dataFolder (string)
    %              
    %       generalModelDescription (EquationsList)
    %       listOfKinetics (array of strings)
    
    properties 
        name string { mustBeNonempty } = "Undefined"
        description string { mustBeNonempty } = "Undefined"
        
        % path to the experimental data
        dataFolder string { mustBeNonempty } = pwd % mustBeFolder
       
        generalModelDescription EquationsList
        listOfKinetics = string.empty
    end
    
    methods
        %% constructor
        function this = FermentationProcessDescription(name, folder, description)
            if nargin ~= 3
                exception = MException('InputParameters:WrongNumber', 'Wrong number of input parameters!');
                throw(exception);           
            end
            
            this.name = name;
            
            this.dataFolder = strcat(pwd, '\Experimental Data\', folder);
            this.description = description;

            this.generalModelDescription = EquationsList();
        end
        
        %% functions for comparing two objects of the class
        function equal = eq(leftObject, rightObject)
            equal = strcmp(leftObject.name, rightObject.name);
        end
        
        function notEqual = ne(leftObject, rightObject)
            notEqual = not(leftObject == rightObject);
        end
        
        %% checks if the current description is a dummy one
        % a fermentation process descripption is empty if the general model
        % description is empty
        function empty = isEmpty(this)
            empty = this.generalModelDescription.isEmpty();
        end
        
        %% find the list of compulsory equations
         function compulsoryEquationsList = getCompulsoryEquations(this)
                       
            compulsoryEquationsList = EquationsList();
            
            numberOfEquations = length(this.generalModelDescription.elements);
            for index = 1: numberOfEquations
                
                currentEquation = this.generalModelDescription.getElementAt(index);
                if currentEquation.isCompulsory
                    compulsoryEquationsList = ...
                        compulsoryEquationsList.addEquation(currentEquation);
                end
                
            end
        end
    end
    
    methods (Access = public, Static = true)
              
        function ecoliFB = createEColiFermentationProcess()
            % initializes the object that corresponds to the E. coli FP
            ecoliFB = FermentationProcessDescription("EcoliFB", "Ecoli", "E. coli MC4110 Fed-batch");

            equationsList = EquationsList();

            dX = EquationDescription(1, "X", "", "dX/dt = mu*X - F/V*X", 1);
            equationsList = equationsList.addEquation(dX);

            dS = EquationDescription(2, "S", "", "dS/dt = -1/Yxs*mu*X + (So - S)*F/V", 1);
            equationsList = equationsList.addEquation(dS);

%             dA = EquationDescription(3, "A", "", "dA/dt = 1/Yax*mu*A - F/V*A", 0);
%             equationsList = equationsList.addEquation(dA);

            dO2 = EquationDescription(4, "O2", "", "dO2/dt = 1/Yox*mu*X + Kla*(O2* - O2) - F/V*O2", 0);
            equationsList = equationsList.addEquation(dO2);

            dV = EquationDescription(5, "V", "", "dV/dt = F", 1);
            equationsList = equationsList.addEquation(dV);

            ecoliFB.generalModelDescription = equationsList;

            ecoliFB.listOfKinetics = ["Monod", "Contois", "Fujimoto"];
        end
               
         function gab = createGlucoicAcidFermentationProcess()
        % initializes the object that corresponds to the Gluconic acid FP
            gab = FermentationProcessDescription("GAB", "Gluconic acid production", "Gluconic acid batch fermentation");

            equationsList = EquationsList();

            dX = EquationDescription(1, "X", "", "dX/dt = Rx", 1);
            equationsList = equationsList.addEquation(dX);

            dGA = EquationDescription(2, "GA", "", "dGA/dt = Rga", 1);
            equationsList = equationsList.addEquation(dGA);

            dS = EquationDescription(3, "S", "", "dS/dt = -1/Yxs*Rx -1/Ysga*Rga", 1);
            equationsList = equationsList.addEquation(dS);

            dO2 = EquationDescription(4, "O2", "", "dO2/dt = 1/Yox*Rga+Kla*(O2*-O2)", 1);
            equationsList = equationsList.addEquation(dO2);

            gab.generalModelDescription = equationsList;

            gab.listOfKinetics = ["Monod"];
        end

    end    
end

